// ignore_for_file: public_member_api_docs

import 'dart:convert';
import 'dart:developer';
import 'dart:io';

import 'package:copy_with_extension/copy_with_extension.dart';
import 'package:flutter/material.dart';
import 'package:healthrian_visualization_support/'
    'healthrian_visualization_support.dart';
import 'package:path_provider/path_provider.dart';

part 'record_button.g.dart';

_$RecordButtonCWProxy get RecordButtonWearEcg =>
    RecordButton(module: BLEManager().wearEcg12.module).copyWith;

_$RecordButtonCWProxy get RecordButtonCardioClick =>
    RecordButton(module: BLEManager().cardioClick.module).copyWith;

_$RecordButtonCWProxy get RecordButtonWearT =>
    RecordButton(module: BLEManager().wearCardioT.module).copyWith;

@CopyWith()
class RecordButton extends StatelessWidget {
  const RecordButton({
    required this.module,
    super.key,
    this.child = const Text('Record'),
  });

  final ModuleInterface module;

  final Widget child;

  @override
  Widget build(BuildContext context) {
    return MaterialButton(
      onPressed: () async {
        final lastEcg = module.ecg?.lastChannel;
        final lastOxi = module.oxi?.lastChannel;
        final appDocDir = await getApplicationDocumentsDirectory();
        final date = DateTime.now().toIso8601String().split('.')[0];
        final path = '${appDocDir.path}/${date.replaceAll(':', '_')}.json';
        final file = File(path);
        log(file.path);

        final data = {
          'device': module.ble.device?.platformName,
          'date': date,
          if (lastEcg != null)
            'ecg': {
              'sampling_rate': module.ecg?.samplingRate,
              'channels': lastEcg.list,
            },
          if (lastOxi != null)
            'oxi': {
              'sampling_rate': module.oxi?.samplingRate,
              'channels': lastOxi.list,
            },
        };

        final dataAsString = jsonEncode(data);

        await file.writeAsString(dataAsString);
      },
      child: child,
    );
  }
}
