import 'package:copy_with_extension/copy_with_extension.dart';
import 'package:fl_chart/fl_chart.dart';
import 'package:flutter/material.dart';

part 'simple_visualization.g.dart';

@CopyWith()
class SimpleVisualization extends StatelessWidget {
  const SimpleVisualization({
    super.key,
    required this.ecgSequence,
    this.time = 10,
    this.rate = 250,
    this.width,
    this.height,
    this.minY = -3,
    this.maxY = 3,

    this.strokeWidth = 1.5,

    this.isTappable = false,
    this.color = Colors.black,
  });

  final List<double> ecgSequence;

  final double time;
  final int rate;
  final double? width;
  final double? height;

  final double minY;
  final double maxY;

  final double strokeWidth;

  final bool isTappable;
  final Color color;

  @override
  Widget build(BuildContext context) {
    final displayLength = (time * rate).round();

    List<FlSpot> ecg = [];
    double index;
    double minX;
    double maxX;

    final redrawingPart = 0;

    index = 0;
    minX = 0;
    maxX = displayLength.toDouble();

    final a = ecgSequence.length - displayLength < 0
        ? 0
        : ecgSequence.length - displayLength;
    final b = ecgSequence.length - redrawingPart;
    final c = ecgSequence.length;

    final remaining = ecgSequence.getRange(a, b);
    final updated = ecgSequence.getRange(b, c);
    final samples = updated.followedBy(remaining);

    for (double i in samples) {
      if (i > maxY) {
        i = maxY;
      } else if (i < minY) {
        i = minY;
      }
      ecg.add(FlSpot(index, i));
      index++;
    }

    return SizedBox(
      width: width,
      height: height,
      child: LineChart(
        LineChartData(
          minX: minX,
          maxX: maxX,
          minY: minY,
          maxY: maxY,
          backgroundColor: Colors.transparent,
          lineTouchData: LineTouchData(enabled: isTappable),
          gridData: FlGridData(
            show: false,
          ),
          titlesData: FlTitlesData(
            show: false,
          ),
          borderData: FlBorderData(
            show: false,
          ),
          lineBarsData: [
            LineChartBarData(
              spots: ecg,
              isCurved: true,
              color: color,
              barWidth: strokeWidth,
              curveSmoothness: 0.15,
              preventCurveOverShooting: true,
              dotData: FlDotData(show: false),
            ),
          ],
        ),
      ),
    );
  }
}
